import pygame
import random
import os

#pygame
from pygame.locals import *
from math import sqrt

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

#utils functions - loading things
from utils import *
#sprite collections
from sprites import *
#player
from player import *
#camera
from camera import *

#vector2d
from vector2d import *

#starfield
from starfield import *

#fader
from fader import *

from safezone import *

from effect import *

MESSAGE_LIFETIME = 4000

class Game:
	
	player = None
	camera = None
	
	collision_groups = None
	
	starfield = None
	background = None
	
	screen = None
	screen_dim = None

	keep_running = None
	last_frame = None
	
	fade_cover = None
	
	friendly_messages = None
	
	safezone = None
	
	sounds = None
	
	planets = None

	#type count
	max_types = {SmallAsteroid: 30, FriendlyShip: 10, LargeAsteroid: 15}
	#max_types = {SmallAsteroid: 10, FriendlyShip: 10, LargeAsteroid: 1}
	current_count = None
	
	clock = pygame.time.Clock()
	
	def DrawDebugText(self):
		#if pygame.font:
		font = pygame.font.Font(None, 14)
		pos_text = "location: " + str(self.player.location)

		text = font.render(pos_text, 1, Color(255, 255, 255))
		textpos = text.get_rect(bottomright=(self.background.get_width(), self.background.get_height()))
		self.background.blit(text, textpos)
		textpos.height += textpos.height
		
		pos_text = "velocity: " + str(self.player.velocity)
		text = font.render(pos_text, 1, Color(255, 255, 255))
		temppos = text.get_rect(bottomright=(self.background.get_width(), self.background.get_height()))
		temppos.bottom -= textpos.height
		textpos = temppos
		self.background.blit(text, textpos)
		bottom_right = textpos.topright
		
		self.clock.tick()
		pos_text = "fps: %.1f" % (self.clock.get_fps())
		text = font.render(pos_text, 1, Color(255, 255, 255))
		temppos = text.get_rect(bottomright=(self.background.get_width(), self.background.get_height()))
		temppos.bottom -= textpos.height
		textpos = temppos
		self.background.blit(text, textpos)
		bottom_right = textpos.topright		
		
		
	#returns a surface.
	def PrepareTitleScreen(self, back_color, text, fore_color, text2 = None):
		title_surface = pygame.Surface(self.screen_dim)
		title_surface.fill(back_color)
		font = pygame.font.Font(None, 36)
		text_surf = font.render(text, 1, fore_color)
		textpos = text_surf.get_rect(center=(title_surface.get_width()/2, title_surface.get_height()/4))
		title_surface.blit(text_surf, textpos)
		if not text2 is None:
			text_surf = font.render(text2, 1, fore_color)
			temppos = text_surf.get_rect(center=(title_surface.get_width()/2, title_surface.get_height()/4))
			temppos.bottom += textpos.height
			textpos = temppos
			title_surface.blit(text_surf, textpos)
		return title_surface
	
	def IsTypeAvailable(self, a_type):
		dsqr_from_origin = self.player.location.MagnitudeSquared()
		if a_type == FriendlyShip:
			return dsqr_from_origin < safe_zone_rad_sqr
		if a_type == SmallAsteroid:
			return dsqr_from_origin < ast_end_rad_sqr and dsqr_from_origin > safe_zone_rad_sqr
		if a_type == LargeAsteroid:
			return dsqr_from_origin < ast_end_rad_sqr and dsqr_from_origin > moderate_ast_rad_sqr

		return False
		
	def SpawnAsteroids(self):
		for a_type in self.collision_groups.keys():
			if len(self.collision_groups[a_type].sprites()) < self.max_types[a_type]:
				if self.IsTypeAvailable(a_type):
					new_sprite = a_type(self.player.location)
					self.collision_groups[a_type].add(new_sprite)
	def AddEscapeMessage(self):
		special_id = -897432
		if special_id not in self.friendly_messages:
			msg = random.choice([ "I'm outta here!",
								"full speed ahead!",
								"let's go!",
								"YEEEEHAWWW!",
								"I wonder what's out there?",
								"go go go!",
								"I wonder how far it goes?",
								"the stars are beautiful!"])
			self.friendly_messages[special_id] = (msg, self.player.location.tup(), pygame.time.get_ticks() + MESSAGE_LIFETIME*random.randrange(2, 4), Color(220, 220, 220))
				
	def AddFriendlyMessage(self, id, location):
		if id not in self.friendly_messages:
			msg = random.choice([ "why would you want to leave?",
								"we have everything here.",
								"all our friends are here.",
								"there is nothing out there for you.",
								"I am happy in Satisfactoria!",
								"it would be foolish to leave."
								])
			rrr = random.randint(70, 90)
			ggg = random.randint(70, 90)
			bbb = random.randint(120, 170)
			self.friendly_messages[id] = (msg, location, pygame.time.get_ticks() + MESSAGE_LIFETIME, Color(rrr, ggg, bbb))
		
	def ShowFriendlyMessages(self):
		dest_surf = self.screen
		#also check for staleness
		time = pygame.time.get_ticks()
		ids_to_remove = []
		
		for (id,msg) in self.friendly_messages.iteritems():
			if msg[2] < time:
				ids_to_remove.append(id)
			else:
				font = pygame.font.Font(None, 20)
				text_surf = font.render(msg[0], 1, msg[3])
				textpos = text_surf.get_rect(center=msg[1])
				dest_surf.blit(text_surf, self.camera.rect_to_screen(textpos))
		for id in ids_to_remove:
			self.friendly_messages.pop(id)
	
	def Init(self):
		pygame.init()
		self.screen_dim = [640, 480]
		
		current_count = {SmallAsteroid: 0, FriendlyShip: 0, LargeAsteroid: 0}
		
		self.screen = pygame.display.set_mode(self.screen_dim)
		pygame.display.set_caption('Satisfactoria')
		pygame.mouse.set_visible(False)
		
		self.starfield = Stars(self.screen_dim)
		
		self.collision_groups = {SmallAsteroid: pygame.sprite.Group(), FriendlyShip: pygame.sprite.Group(), LargeAsteroid: pygame.sprite.Group()}
			
		self.player = Player(Vector2d((0,0)))
		
		self.camera = Camera(self.player.location.copy(), 640, 480, 640, 480)
		
		self.keep_running = True
		self.last_frame = pygame.time.get_ticks()
		
		self.fade_cover = Fader(self.PrepareTitleScreen((255, 255, 255), "Not far enough.", (80, 80, 80)))
		
		self.friendly_messages = {}
		self.safezone = Safezone(Vector2d((1.9*sqrt(safe_zone_rad_sqr), 1.9*sqrt(safe_zone_rad_sqr))))
		
		self.sounds = {"thrusting": Effect("thruster.wav"), "explode": Effect("explode.wav"), "win": Effect("win.wav")}
		
		self.planets = [Planet("planet_%s.png" %(x), (x)) for x in ["pink", "purple", "teal", "bordeaux", "red", "yellow", "blue", "orange"]]
		circ_per = field_circ / len(self.planets)
		for i in range(0, len(self.planets)):
			angle = i*360/len(self.planets)
			circ_per
			self.planets[i].image = pygame.transform.scale(self.planets[i].image, [int(circ_per), int(circ_per/6)])
			self.planets[i].image = pygame.transform.rotate(self.planets[i].image, -angle)
			self.planets[i].rect = self.planets[i].image.get_rect(center=(-math.sin(math.radians(angle)) * arrived_rad, math.cos(math.radians(angle)) *arrived_rad))
		
	def MainLoop(self):
		while (self.keep_running):

			#deal with time
			time = pygame.time.get_ticks()
			elapsed_time = time - self.last_frame
			self.last_frame = pygame.time.get_ticks()
			
			event = pygame.event.poll()
			if ((event.type == pygame.QUIT) or (event.type == KEYDOWN and event.key == 27)):
				self.keep_running = False
				break;
			else:
				self.player.process_input(pygame.key.get_pressed(), elapsed_time)
			
			#do player update
			self.player.update(elapsed_time)
			if self.player.rail_mode:
				self.AddEscapeMessage()
			
			if self.player.thrusting:
				self.sounds["thrusting"].Play(time, True)
			else:
				self.sounds["thrusting"].Stop()
				
			for sound in self.sounds.values():
				sound.Update(time)
			
			self.camera.set_target(self.player.location.copy())
			self.camera.update(elapsed_time)

			#background
			self.background = self.starfield.GetSurface(self.player.velocity.Div(60))
			
			self.DrawDebugText()
			
			self.screen.blit(self.background, (0, 0))
			self.screen.blit(self.safezone.GetSurface(), self.camera.rect_to_screen(self.safezone.GetRect()))
			
			#lose condition
			for group in self.collision_groups.values():
				group.update(elapsed_time, self.player.location.copy())
				for b in group.sprites():
					if self.player.has_life:
						if pygame.sprite.collide_mask(b, self.player):
							if(b.IsFriendly()):
								self.AddFriendlyMessage(b.id, b.rect.center)
							else:
								self.player.kill_player(True)
								self.sounds["explode"].Play(time, False)
								self.sounds["thrusting"].Stop()
					self.screen.blit(b.image_for_display, self.camera.rect_to_screen(b.rect))
					
			#win condition
			if self.player.has_life:
				for planet in self.planets:
					if pygame.sprite.collide_rect(planet, self.player):
						if pygame.sprite.collide_mask(planet, self.player):
							self.fade_cover = Fader(self.PrepareTitleScreen((255, 255, 255),
									"You have discovered the " + planet.name + " planet.",
									(0, 0, 0), "You thrive and prosper."))
							self.player.kill_player(False)
							self.sounds["win"].Play(time, False)
							self.sounds["thrusting"].Stop()
				
			if not self.player.has_life :
				self.fade_cover.StartFading()

			self.fade_cover.Update(elapsed_time)
			
			self.SpawnAsteroids()
			
			self.ShowFriendlyMessages()
			
			
			for planet in self.planets:
				self.screen.blit(planet.image, self.camera.rect_to_screen(planet.rect))

			#place player
			if self.player.Deathed:
				self.screen.blit(self.player.splode_image, self.camera.rect_to_screen(self.player.rect))
			else:
				self.screen.blit(self.player.image, self.camera.rect_to_screen(self.player.rect))
			
			#LAST thing to go up
			if self.fade_cover.IsFading():
				self.screen.blit(self.fade_cover.GetSurface(), (0,0))

			pygame.display.update()