
import pygame
import math

#utils functions - loading things
from utils import *

from vector2d import *

MAX_VELOCITY = 80

class Player(pygame.sprite.Sprite):
	image = None
	original = None
	rect = None
	
	location = None
	keys_down = None
	initial_position = None

	velocity = None
	heading = None
	
	has_life = None
	thrusting = None
	
	rail_mode = None
	rail_heading = None
	
	splode_image = None
	
	Deathed = None
	
	rocket_image = None
	rocket_rect = None
	
	def __init__(self, initial_position):
		pygame.sprite.Sprite.__init__(self)
		
		if Player.image is None:
			Player.image, Player.rect = utils.load_image("ship3.bmp", -1)
		self.splode_image, splode_rect = utils.load_image("splode.png", -1)
		self.rocket_image, self.rocket_rect = utils.load_image("rocket.bmp", -1)
		
		self.image = Player.image
		self.original = self.image
		self.rect = Player.rect.copy()
		
		self.has_life = True
		
		self.initial_position = initial_position.copy()
		
		self.reset()
		self.thrusting = False
		self.Deathed = False
		
	def reset(self):
		#these are in world coords - not screen!
		self.location = self.initial_position.copy()
		self.velocity = Vector2d((0,0))
		self.heading = 0
		self.rail_mode = False
		self.rail_heading = 0
		
	def RailMode(self):
		if self.rail_mode: return
		self.rail_mode = True
		#get current direction
		dot = Vector2d((0, -1)).Dot(self.velocity.Normalize())
		was_x_negative = self.velocity.x < 0
		#print was_x_negative

		if dot > 1 : dot = 1
		if dot < -1 : dot = -1
		curr_head = -math.degrees(math.acos(dot));
		if was_x_negative: curr_head = -curr_head
		self.heading = curr_head
		self.rail_heading = curr_head
	
	def kill_player(self, deathed):
		self.thrusting = False
		self.has_life = False
		self.velocity = Vector2d((0,0))
		self.Deathed = deathed
	
	def apply_velocity(self, time_elapsed):
		self.location += self.velocity.Mul(time_elapsed / 1000.)
		if self.location.MagnitudeSquared() > safe_zone_rad_sqr*1.1:
			self.RailMode()
		
	def thrust(self, amount):
		if self.has_life:
			self.thrusting = True
		
		self.velocity += Vector2d((math.sin(math.radians(-self.heading)) * amount, -math.cos(math.radians(self.heading)) * amount))
		mag = self.velocity.MagnitudeSquared()

		if mag > MAX_VELOCITY**2:
			self.velocity = self.velocity.Normalize().Mul(MAX_VELOCITY)
		
	def turn(self, amount):
		self.heading += amount
		if self.rail_mode:
			max_spread = 40
			if self.heading < (self.rail_heading - max_spread) : self.heading =(self.rail_heading - max_spread)
			if self.heading > (self.rail_heading + max_spread) : self.heading =(self.rail_heading + max_spread)
		
	def process_input(self, key, time_elapsed):
		angle_multiplier = 20 / 100.;
		thrust_multiplier = 15 / 100.;
		
		self.thrusting = False

		if key[pygame.K_LEFT]:
			self.turn(angle_multiplier*time_elapsed)
		if key[pygame.K_RIGHT]:
			self.turn(-angle_multiplier*time_elapsed)

		if self.rail_mode:
			self.thrust(thrust_multiplier*time_elapsed)
		else:
			if key[pygame.K_UP]:
				self.thrust(thrust_multiplier*time_elapsed)
			if key[pygame.K_DOWN]:
				self.thrust(-thrust_multiplier*time_elapsed)
	
	def update(self, time_elapsed):
		self.apply_velocity(time_elapsed)
		self.rect.center = self.location.tup()
		self.image = pygame.transform.rotate(self.original, 0)
		if self.thrusting:
			self.image.blit(self.rocket_image, (4 , 30))
			self.image.blit(self.rocket_image, (22 , 30))
		self.image = pygame.transform.rotate(self.image, self.heading)
		self.rect = self.image.get_rect(center=self.rect.center)