
from pygame import *

import random

#utils functions - loading things
from utils import *

from vector2d import *

class CollidableSprite(pygame.sprite.Sprite):
	image = None
	rect = None;
	
	speed = None
	heading = None
	location = None
	
	spin = None
	rotation = None
	image_for_display = None
	id = None
	last_id = -1
	
	def __init__(self, player_position):
		if CollidableSprite.last_id == -1:
			CollidableSprite.last_id = 1
		
		self.id = CollidableSprite.last_id
		CollidableSprite.last_id += 1

		
		pygame.sprite.Sprite.__init__(self)
		self.speed = random.randrange(1, self.MaxSpeed())
		self.heading = random.randrange(0, 360)

		rand_theta = random.randrange(0, 360)
		spawndis = random.randrange(self.MinSpawnDistance(), self.MaxSpawnDistance())
		self.location = player_position + Vector2d((spawndis*math.sin(-math.radians(rand_theta)), spawndis*math.cos(math.radians(rand_theta))))

		self.spin = random.randrange(1, self.MaxSpin()) * random.choice([-1, 1])
		self.rotation = 0
		#self.image_for_display = self.image.copy()
		
	def update(self, elapsed_time, player_pos):
		amount = elapsed_time / 20.
		self.location += Vector2d((math.sin(math.radians(-self.heading)) * amount * self.speed, -math.cos(math.radians(self.heading)) * amount * self.speed))
		self.rotation += self.spin*elapsed_time / 100.
		self.image_for_display = pygame.transform.rotate(self.image, self.rotation)
		self.rect.center = self.location.tup()
		self.rect = self.image_for_display.get_rect(center=self.rect.center)
		
		#check extreme distances
		if (player_pos - self.location).MagnitudeSquared() > 1000**2:
			self.kill()


class SmallAsteroid(CollidableSprite):

	def MaxSpeed(self):
		return 3

	def MaxSpin(self):
		return 20
		
	def MinSpawnDistance(self):
		return 1000
	
	def MaxSpawnDistance(self):
		return 1001
		
	def IsFriendly(self):
		return False

	def __init__(self, player_position):
		CollidableSprite.__init__(self, player_position)
		
		if SmallAsteroid.image is None:
			SmallAsteroid.image, SmallAsteroid.rect = utils.load_image("small_asteroid.png", -1)
			
		self.image = SmallAsteroid.image
		
	def update(self, elapsed_time, player_pos):
		CollidableSprite.update(self, elapsed_time, player_pos)
		if self.location.MagnitudeSquared() <= safe_zone_rad_sqr :
			#get current heading vector
			a = Vector2d((math.sin(math.radians(-self.heading)), -math.cos(math.radians(self.heading)))).Normalize()
			b = (self.location).Normalize()
			dot = a.Dot(b)
			if(dot > 1) : dot = 1
			if(dot < -1) : dot = -1
			theta = math.degrees(math.acos(dot))
			correction = random.randrange(3, 100)
			self.heading += theta/float(correction)
	

				
class FriendlyShip(CollidableSprite):

	def MaxSpeed(self):
		return 2
	
	def MaxSpin(self):
		return 2
		
	def MinSpawnDistance(self):
		return 100
	
	def MaxSpawnDistance(self):
		return 500
		
	def IsFriendly(self):
		return True

	def __init__(self, player_position):
		CollidableSprite.__init__(self, player_position)
		
		if FriendlyShip.image is None:
			image_chosen = random.choice(["friendly_%d.bmp" % (x) for x in range(1, 2)])
			FriendlyShip.image, FriendlyShip.rect = utils.load_image(image_chosen, -1)
			
		self.image = FriendlyShip.image

	def update(self, elapsed_time, player_pos):
		CollidableSprite.update(self, elapsed_time, player_pos)
		if self.location.MagnitudeSquared() >= safe_zone_rad_sqr :
			#get current heading vector
			a = Vector2d((math.sin(math.radians(-self.heading)), -math.cos(math.radians(self.heading)))).Normalize()
			b = (Vector2d((0,0)) - self.location).Normalize()
			dot = a.Dot(b)
			if(dot > 1) : dot = 1
			if(dot < -1) : dot = -1
			theta = math.degrees(math.acos(dot))
			correction = random.randrange(3,9 )
			self.heading += theta/float(correction)
		
class LargeAsteroid(CollidableSprite):

	def MaxSpeed(self):
		return 4

	def MaxSpin(self):
		return 15
		
	def MinSpawnDistance(self):
		return 1000
	
	def MaxSpawnDistance(self):
		return 1001
		
	def IsFriendly(self):
		return False

	def __init__(self, player_position):
		CollidableSprite.__init__(self, player_position)
		
		if LargeAsteroid.image is None:
#			image_chosen = random.choice(["large_a%d.bmp" % (x) for x in range(1, 2)])
			LargeAsteroid.image, LargeAsteroid.rect = utils.load_image("large_asteroid.bmp", -1)
			
		self.image = LargeAsteroid.image

class Planet(pygame.sprite.Sprite):
	image = None
	rect = None;

	rotation = None
	image_for_display = None
	
	name = None
	
	def __init__(self, filename, name):
		pygame.sprite.Sprite.__init__(self)
		
		#if Planet.image is None:
		self.image, self.rect = utils.load_image(filename, -1)
		
		self.name = name
		
		self.rotation = 0
	
	def SetRect(self, cenx, ceny):
		self.rect.center = (cenx, ceny)
		
	# def update(self, elapsed_time, player_pos):
		# amount = elapsed_time / 20.
		# self.location += Vector2d((math.sin(math.radians(-self.heading)) * amount * self.speed, -math.cos(math.radians(self.heading)) * amount * self.speed))
		# self.rotation += self.spin*elapsed_time / 100.
		# self.image_for_display = pygame.transform.rotate(self.image, self.rotation)
		# self.rect.center = self.location.tup()
		# self.rect = self.image_for_display.get_rect(center=self.rect.center)
		
		# #check extreme distances
		# if (player_pos - self.location).MagnitudeSquared() > 1000**2:
			# self.kill()